from django.db import models
from django.contrib.auth import get_user_model


class CompanySector(models.Model):
    name = models.CharField(max_length=100)

    def __str__(self):
        return self.name

class CompanySubSector(models.Model):
    name = models.CharField(max_length=100)

    def __str__(self):
        return self.name

class CompanySegment(models.Model):
    name = models.CharField(max_length=100)

    def __str__(self):
        return self.name

class ListingSegment(models.Model):
    name = models.CharField(max_length=100)

    def __str__(self):
        return self.name

class Stock(models.Model):

    STOCK_TYPE_CHOICES = (
    ('ON', 'Ordinary'),
    ('PN', 'Preffered'),
        )

    symbol = models.CharField(max_length=8)
    stock_type = models.CharField(max_length=2, choices=STOCK_TYPE_CHOICES)
    description = models.TextField(null=True, blank=True)

    def __str__(self):
        return self.symbol


class Company(models.Model):


    CATEGORY_CHOICES = (
    ('AL', 'Alerta'),
    ('NC', 'Não Classificada'),
    ('EQ', 'Equilibrada'),
    ('CL', 'Cíclica'),
        )


    name = models.CharField(max_length=100)
    symbol = models.CharField(max_length=4)
    stocks = models.ManyToManyField(Stock)
    cnpj = models.CharField(max_length=19)
    activity = models.CharField(max_length=300)
    sector = models.ForeignKey(CompanySector, on_delete=models.CASCADE)
    subsector = models.ForeignKey(CompanySubSector, on_delete=models.CASCADE)
    market_segment = models.ForeignKey(CompanySegment, on_delete=models.CASCADE)
    listing_segment = models.ForeignKey(ListingSegment, on_delete=models.CASCADE)
    market_value = models.CharField(max_length=100, null=True, blank=True)
    ev = models.CharField(max_length=100, null=True, blank=True)
    bookkeeper = models.CharField(max_length=100)
    majority_shareholder = models.CharField(max_length=100)
    site = models.URLField()
    headquarters= models.CharField(max_length=150)
    bovespa_page = models.URLField()
    shareholders = models.PositiveIntegerField()
    category = models.CharField(max_length=2, choices=CATEGORY_CHOICES)
    ibov = models.BooleanField(default=False)
    next_balance = models.DateField(null=True, blank=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name_plural = "Companies"


class Broker(models.Model):
    name = models.CharField(max_length=100)

    def __str__(self):
        return self.name

class StockProfile(models.Model):
    name = models.CharField(max_length=100)
    user = models.ForeignKey(get_user_model(), on_delete=models.CASCADE)


    def __str__(self):
        return self.name


class Transaction(models.Model):
    OPERATION_TYPE = (
        ('BUY', 'Buy'),
        ('SEL', 'Sell'),
        ('DIV', 'Dividends'),
        ('SPL', 'Split'),
        ('AGR', 'Aggregation'),)
    user = models.ForeignKey(get_user_model(), on_delete=models.CASCADE)
    profile = models.ForeignKey(StockProfile, on_delete=models.CASCADE)
    stock = models.ForeignKey(Stock, on_delete=models.CASCADE)
    operation_type = models.CharField(null=True,blank=True,max_length=3, choices=OPERATION_TYPE)
    operation_date = models.DateField()
    shares = models.PositiveIntegerField()
    price = models.DecimalField(decimal_places=9,max_digits=15)
    commission = models.DecimalField(decimal_places=2,max_digits=6)
    broker = models.ForeignKey(Broker, on_delete=models.CASCADE)
    notes = models.TextField(null=True,blank=True)

    def __str__(self):
        return '%s' % (str(self.id))

class Balance(models.Model):
    company = models.ForeignKey(Company, on_delete=models.CASCADE)



from django.contrib import admin
from .models import Loan, FixedIncome, InvestmentFund

from .stocks.models import Stock, ListingSegment, CompanySector, CompanySubSector, CompanySegment, Broker, Company, Transaction, StockProfile
# Register your models here.

class FilterUserAdmin(admin.ModelAdmin):

	def save_model(self, request, obj, form, change):
		if obj.user == None:
			obj.user = request.user
		obj.save()

	def get_queryset(self, request):
		#qs = admin.ModelAdmin.queryset(self, request)
		qs = super(FilterUserAdmin, self).get_queryset(request)
		if request.user.is_superuser:
			return qs
		return qs.filter(user=request.user)

	def has_change_permission(self, request, obj=None):
		if not obj:
			return True
		if request.user.is_superuser or obj.user == request.user:
			return True
		else:
			return False

	# Set delete permissions as change permissions. If user can change, can also delete
	has_delete_permission = has_change_permission


class LoanAdmin(admin.ModelAdmin):
	list_display = ('date', 'loan_name', 'person', 'value')
	search_fields = ('loan_name', 'person')


class FixedIncomeAdmin(admin.ModelAdmin):
	list_display = ('product_type', 'signer', 'start_date', 'due_date', 'position')
	search_fields = ('product_type', 'signer')


class InvestmentFundAdmin(admin.ModelAdmin):
	list_display = ('ativo', 'fund_type', 'net_value')
	search_fields = ('ativo', 'fund_type',)


class TransactionAdmin(admin.ModelAdmin):
	list_display = ('stock', 'shares', 'price', 'operation_type', 'operation_date','broker',)
	search_fields = ('stock',)
	list_filter = ('stock','operation_type','profile','broker','user')


class BrokerAdmin(admin.ModelAdmin):
	list_display = ('name',)
	search_fields = ('name',)


class CompanyAdmin(admin.ModelAdmin):
	list_display = ( 'symbol', 'name', 'sector','category', 'shareholders', 'ibov', 'next_balance',)
	search_fields = ('name','symbol','category', )
	list_filter = ('symbol','sector','ibov', 'category',)

class CompanySectorAdmin(admin.ModelAdmin):
	list_display = ('name',)
	search_fields = ('name',)

class CompanySubSectorAdmin(admin.ModelAdmin):
	list_display = ('name',)
	search_fields = ('name',)

class CompanySegmentAdmin(admin.ModelAdmin):
	list_display = ('name',)
	search_fields = ('name',)

class ListingSegmentAdmin(admin.ModelAdmin):
	list_display = ('name',)
	search_fields = ('name',)


class StockAdmin(admin.ModelAdmin):
	list_display = ('symbol', 'stock_type')
	search_fields = ('symbol',)

class StockProfileAdmin(FilterUserAdmin):
	list_display = ('name', 'user' )
	search_fields = ('name',)
	#list_filter = (('user', admin.BooleanFieldListFilter))





#Investments Registrations
admin.site.register(Loan, LoanAdmin)
admin.site.register(FixedIncome, FixedIncomeAdmin)
admin.site.register(InvestmentFund, InvestmentFundAdmin)

# Stocks Registrations
admin.site.register(Stock, StockAdmin)
admin.site.register(ListingSegment, ListingSegmentAdmin)
admin.site.register(CompanySector, CompanySectorAdmin)
admin.site.register(CompanySubSector, CompanySubSectorAdmin)
admin.site.register(CompanySegment, CompanySegmentAdmin)
admin.site.register(Broker, BrokerAdmin)
admin.site.register(Company, CompanyAdmin)
admin.site.register(Transaction, TransactionAdmin)
admin.site.register(StockProfile, StockProfileAdmin)



"""Investments Model."""
from django.db import models
from django.contrib.auth import get_user_model


class Loan(models.Model):
    loan_name = models.CharField(max_length=100)
    description = models.TextField(blank=True, null=True)
    date = models.DateField()
    value = models.DecimalField(blank=True, null=True, decimal_places=2, max_digits=50)
    person = models.CharField(max_length=150)

    def __str__(self):
        return "%s - %s" % (self.person, self.loan_name)


class FixedIncome(models.Model):

    product_type = models.CharField(max_length=50)
    signer = models.CharField(max_length=100)
    start_date = models.DateField()
    due_date = models.DateField()
    validity_date = models.DateField()
    tax = models.CharField(max_length=50)
    available_amount = models.DecimalField(blank=True, null=True, decimal_places=2, max_digits=50)
    guarantee = models.DecimalField(blank=True, null=True, decimal_places=2, max_digits=50)
    start_amount = models.DecimalField(blank=True, null=True, decimal_places=2, max_digits=50)
    position = models.DecimalField(blank=True, null=True, decimal_places=2, max_digits=50)

    def __str__(self):
        return "%s - %s" % (self.product_type, self.signer)


class InvestmentFund(models.Model):
    FUND_TYPE_CHOICES = (
    ('RF', 'Renda Fixa'),
    ('MM', 'Multimercado'),
    ('FI', 'Fundo Imobiliário'),)

    ativo = models.CharField(max_length=100, unique=True)
    quota_date = models.DateField()
    quota_value = models.DecimalField(blank=True, null=True, decimal_places=8, max_digits=50)
    quote_amount = models.DecimalField(blank=True, null=True, decimal_places=8, max_digits=50)
    position = models.DecimalField(blank=True, null=True, decimal_places=2, max_digits=50)
    net_value = models.DecimalField(blank=True, null=True, decimal_places=2, max_digits=50)
    fund_type = models.CharField(max_length=2, choices=FUND_TYPE_CHOICES)

    def __str__(self):
        return self.ativo



import datetime as dt
import matplotlib.pyplot as plt
from matplotlib import style
import pandas as pd
import pandas_datareader.data as web

style.use('ggplot')

start = dt.datetime(2000,1,1)
end = dt.datetime(2017,12,31)

#dataframe
df = web.DataReader('LREN3.SA', 'yahoo', start, end)

# Print to Debug
#print(df.tail(10))


# Save to CSV file
df.to_csv('tmp/stocks/LREN.csv')


# df = pd.read_csv('tmp/stocks/LREN.csv', parse_dates=True, index_col=0)
